<html>
<head>
<title>Cluster status</title>
<% 
#############################################################################
# Check for auto-refresh
set refresh 0
asp::decode_input
if {[info exists asp::input(refresh)] && $asp::input(refresh)=="yes" } {
    set refresh 1
    asp::write "<meta http-equiv=Refresh content=\"2; $request::env(SCRIPT_NAME)?refresh=yes\">"
}


#############################################################################
## Attempts to connect with a timeout, either errors or returns a 
## socket channel (blocking)
##
proc connect_timeout { host port timeout } {
    global connect_timeout ;# global table for our state info
    set connect_timeout(connected) ""
    set connect_timeout(socket) [socket -async $host $port]
    set t [expr $timeout*1000]
    set connect_timeout(timeoutid) [after $t { 
	close $connect_timeout(socket)
	set connect_timeout(connected) "no"
    }]
    fileevent $connect_timeout(socket) writable { 
	after cancel $connect_timeout(timeoutid)
	set connect_timeout(connected) "yes"
    }
    # Now wait if we didn't manage it straight away....
    if { $connect_timeout(connected) == "" } { 
	vwait connect_timeout(connected) 
    }
    if { $connect_timeout(connected) == "yes" } { 
	# Ensure we are back blocking again
	fconfigure $connect_timeout(socket) -blocking yes 
	return $connect_timeout(socket) 
    }
    error "Connect timeout"
}

#############################################################################
##
## Draws a pictorial representation of the current server status
##
## Requries
##    zap functionality
##    single-black.gif
##    server.gif
##    deadserver.gif

proc draw_status {} {

    #
    ## Registry settings
    #
    # List of frontend machines
    set reg_frontend "conf!web!cluster!frontends"
    # List of backend machines
    set reg_backend  "conf!web!cluster!hosts"
    # Which one is master
    set reg_master   "dynamic!cluster!master"
    # Where the status of a backend machine lives
    set reg_status   "dynamic!cluster!status"
    # Where the status of our slave is
    set reg_slaveok  "dynamic!cluster!slaveok"

    # Work out if the cluster master frontend server is running
    set cluster_running 0
    set host [zap::regget "dynamic!cluster!master"]
    if { $host != "" } { set port [zap::regget "dynamic!apps!cluster!$host"] }
    if { $host != "" && $port != "" } {
	# Try to connect to host:port to work out what its running
	catch { 
	    set s [ connect_timeout $host $port 2 ]
	    # If we get this far, its good
	    incr cluster_running
	}
    }

    if { !$cluster_running } { asp::write "Cluster is not running" }

    # ----------------------------------------------------------
    # Returns 1 if the backend machine is functional, else 0
    # A backend machine is considered functional if its response time < 10000
    
    proc isworking { backend } {
	uplevel {if { !$cluster_running } { return 1 }}
	set status [zap::regget "$backend"]
	# status is percent_load,response_time,{up|down}
	set dlist [split $status ,]
	if {[lindex $dlist 2] == "down" } { return 0 }
	return 1
    }

    # ----------------------------------------------------------
    # Returns the response time in ms
    proc getstats { backend } {
	uplevel {if { !$cluster_running } { return "" }}
	set status [zap::regget "$backend"]
	# status is percent_load,response_time
	if { $status == "" } { return "" }
	set dlist [split $status ,]
	set l [lindex $dlist 0]
	set r [lindex $dlist 1]
	return "$l % workload<br>$r ms"
    }
    
    # ----------------------------------------------------------
    # Returns the hostname of a machine given its full name
    
    proc hostname { fullname } {
	return [lindex [split $fullname .] 0]
    }
    
    # ----------------------------------------------------------
    
    ## Get the settings
    set frontends [lsort -dictionary [zap::reglist $reg_frontend]]
    set backends  [lsort -dictionary [zap::reglist $reg_backend]]
    set master    [zap::regget $reg_master]
    set slaveok   [zap::regget $reg_slaveok]

    # If the cluster isn't running, make it look like everything is up
    if { !$cluster_running } {
	set slaveok "yes"
	if { $master == "" } { set $master [lindex $frontends 1] }
    }

    # There should be one or two frontend machines
    if { [llength $frontends] == 0 || [llength $frontends] > 2 } {
	zap::error "Invalid frontend configuration"
    }
    
    ## Now lets think about our table
    #
    # |-------------------------------------------------|
    # |..........|frontend1|........|frontend2|.........|
    # |-------------------------------------------------|
    # |backend1|...|backend2|....|backend3|....|backend4|
    # ---------------------------------------------------
    
    # Work out how many columns we need
    set max [llength $frontends]
    if { [llength $backends ] > $max } { set max [llength $backends] }
    set columns $max

    # Start our table
    asp::write "<table border=0 cellspacing=0 cellpadding=0>\n"
    
    # Write out the columns
    asp::write "<tr>"
    for {set i 0} {$i < $columns} {incr i} {asp::write "<td>&nbsp;</td>" }
    asp::write "</tr>\n"

    # We need a router!
    asp::write "<tr><td colspan=$columns align=middle>
    <img src=router.gif><br>
    <img src=single-black.gif height=20 width=1></td></tr>\n"

    # Now our bus
    asp::write "<tr bgcolor=#000000><td height=1 colspan=$columns>
    <img src=single-black.gif width=1 height=1></td></tr>\n"

    # Now the bus onto frontend connections
    asp::write "<tr>"
    foreach i $frontends {
	set span [expr $columns/2]
	if { [llength $frontends] == 1 } { set span $columns }
	set msg "&nbsp;"
	if { $i == $master } { 
	    set msg "<img src=single-black.gif width=1 height=20>"
	}
	asp::write "<td colspan=$span align=middle>$msg</td>"
    }
    asp::write "</tr>\n"

    # Write our frontend row
    asp::write "<tr>"
    foreach i $frontends {
	set span [expr $columns/2]
	if { [llength $frontends] == 1 } { set span $columns }
	set img "server.gif"
	if { $i != $master && $slaveok == "no" } { 
	    set img "deadserver.gif" }
	asp::write "<td colspan=$span align=middle>
	<img src=$img hspace=20><br>
	<font size=-1>[hostname $i]</font>
	</td>"
    }
    asp::write "</tr>\n"
    
    # Now our frontend onto bus row
    asp::write "<tr>"
    foreach i $frontends {
	set span [expr $columns/2]
	if { [llength $frontends] == 1 } { set span $columns }
	set msg "<img src=single-black.gif height=20 width=1>"
	if { $i != $master && $slaveok == "no" } { set msg "&nbsp;" }
	asp::write "<td colspan=$span align=middle>$msg</td>"
    }
    asp::write "</tr>\n"
    
    # Now our bus
    asp::write "<tr bgcolor=#000000><td height=1 colspan=$columns>
    <img src=single-black.gif width=1 height=1></td></tr>\n"
    
    # Now the backend onto bus row
    asp::write "<tr>"
    foreach i $backends {
	# Check if this backend is working
	set msg "&nbsp;"
	if { [isworking "$reg_status!$i"]  } { 
	    set msg "<img src=single-black.gif height=20 width=1>" 
	}
	asp::write "<td align=middle>$msg</td>"
    }
    
    asp::write "</tr>\n"
    
    # Now the backend row
    asp::write "<tr>"
    foreach i $backends {
	set img "server.gif"
	if { ![isworking "$reg_status!$i"] } { set img "deadserver.gif" }
	asp::write "<td align=middle valign=top>
	<img src=$img hspace=30><br>
	<font size=-1>[hostname $i]<br>
	[getstats \"$reg_status!$i\"]</font></td>"
    }
    asp::write "</tr>\n"
    
    # All done
    asp::write "</table>"
}
##############################################################################

%>
</head>

<body bgcolor="#ffffff" link=#990066 vlink=#990066 alink=#990066>

<table>
<tr>
 <td valign=top>
  <img src="/apps/web/title.gif" hspace=10 alt="">
  <p align=right>

  <a href="/apps/admin/docsystem/" target="help" onClick=
     "window.open('','help','resizable=1,width=640,height=500'); return true;">
        <img src="/apps/web/docs.gif" 
        alt="[docs]" align=bottom border=0 hspace=0></a>

  <a href="/apps/web/"><img src="/apps/web/back.gif" 
        alt="[back]" align=bottom border=0 hspace=0></a>
        <a href="/apps/web/"><img src="/apps/web/home.gif" alt="[Controller]" 
        align=bottom border=0></a>
  </td>
  <td valign=top>


<br>
<img src="title-clusterstatus.gif" alt="Cluster Status">

<ul>
<%
draw_status
asp::write "<p>"

if { $refresh } {
    asp::write "<a href=\"$request::env(SCRIPT_NAME)\">
    Disable auto-refresh</a>"
} else {
    asp::write "<a href=\"$request::env(SCRIPT_NAME)?refresh=yes\">
    Enable auto-refresh</a>"
}
%>
&nbsp;&nbsp;&nbsp;&nbsp;
<a href="settings.asp">Configuration</a>
</ul>

</td>
</tr>
</table>

</body>
</html>
