<%
## Adds a new website to the webserver
## If successful, redirects to controller, else displays error
##
## Arguments
##    name 
##    ip_name  (server:port)
##    docroot
##    clone  (!default means skel)
##
# This routine will add a new website
proc add { name docroot ip_name port clone } {
    set br "conf!web!servers!$name"
    reg::addbranch "$br"

    # Work out what to copy from
    set from "conf!web!skel";
    if { $clone != "!default" } { set from "conf!web!servers!$clone" }

    reg::copy "$from" "$br"
    reg::add "$br!ip_name" $ip_name text
    reg::add "$br!port"    $port    integer
    reg::add "$br!docroot" $docroot text
}

# Decode and check the input before calling our add routine
zap::getrequest { name ip_name docroot }
set name    $asp::input(name)
set ip_name $asp::input(ip_name)
set docroot $asp::input(docroot)
set clone   $asp::input(clone)

if { [ string match "* *" $name ] } {
    zap::error "Server name may not contain spaces"
}

if { [ string match "* *" $ip_name ] } {
    zap::error "Website URL may not contain spaces"
}

# Process ip_name field, it may have a port number etc.
set port 80
regexp (.*):(\[0-9]+) $ip_name tmp ip_name port

# Check our setting are reasonable
if { $ip_name == "" }          { zap::error "Blank website URL" }
if { $port    == "" }          { zap::error "Blank port" }
if { $port    == "0" }         { zap::error "Invalid port" }
if { $name    == "" }          { zap::error "Blank name" }

# If we are clustering, we don't check if the docroot exists, as it
# may only exist on the back-end machines...
set cluster 0
if { [ zap::regget "conf!web!cluster!enabled" ] == "yes" } { set cluster 1 }
if { $cluster == 0 } {
    if { [file exists $docroot] == 0 } {
	zap::error "Document root directory <b>$docroot</b> does not exist" 
    }
    if { [file isdirectory $docroot ] == 0 } {
	zap::error "Document root <b>$docroot</b> is not a directory!" 
    }
}

# Check if we already have one 
if { [ zap::reglist "conf!web!servers!$name" ] != "" } { 
    zap::error "Server <b>$name</b> already exists"
}

# Lets do it
if [ catch { add $name $docroot $ip_name $port $clone } result ] {
    if { [ string match "*Access denied" $result ] } {
	asp::authenticate "Registry update"
    }
    zap::error "Unable to add server <b>$name</b>: $result"
}

# It succceded
asp::redirect "/apps/web/"
%>