#!/bin/sh



# ------------------------------------------
# Gets a registry setting

getsetting() 
{
  VALUE=`echo 'get conf!web!servers!'$SERVERNAME'!'$1 | regcmd -`
}


# ------------------------------------------
# checks that a module is enabled

moduleenabled()
{
    getsetting 'modules!'$1'!enabled' 2>/dev/null
    if [ $? -eq 0 ]
    then
	if [ "$VALUE" = "yes" ] 
	then
	    return 0
        fi
    fi
    return 1
}


# ------------------------------------------
# errors if a module isn't enabled

assert_moduleenabled()
{
    moduleenabled $1
    if [ $? -ne 0 ]
    then
       echo "FATAL: $1 module is not enabled for this server"
       exit;
    fi
    return 0
}


# ------------------------------------------
# check a UNIX user exits

checkuser()
{
  if [ "$1" = "" ] 
  then
	return 1
  fi
  cut -d':' -f1  /etc/passwd | grep '^'$1'$' > /dev/null
  return $?
}


# ------------------------------------------
# Check a group exists

checkgroup()
{
  if [ "$1" = "" ] 
  then
	return 1
  fi
  cut -d':' -f1  /etc/group | grep '^'$1'$' > /dev/null
  return $?
}


# ------------------------------------------
# gets and confirms a password from the user

getpassword()
{
 prompt=$1
 VALUE=""
 until [ "$VALUE" != "" ]
 do
    printf "$prompt"
    stty -echo
    read password1
    stty echo
    echo
    printf "Re-enter $prompt"
    stty -echo
    read password2
    stty echo
    echo 
    if [ "$password1" = "$password2" ]
    then
        VALUE=$password1
    fi
 done

 return 1
}


# ------------------------------------------
# checks a server exists

checkserver()
{
  a=`echo 'get conf!web!servers!'$1'!port' | regcmd - 2> /dev/null`
  return $?
}


# ------------------------------------------
# The main code

cat << EOM

Zeus Technology - Express FrontPage Setup
Copyright (C) - Zeus Technology, 1998

EOM

# make sure regcmd is in our path
PATH=$ZEUSHOME/bin:/usr/local/frontpage/currentversion/bin:$PATH
export PATH

# check frontpage directory exists
if [ ! -d /usr/local/frontpage ]
then
  echo "FATAL: Microsoft FrontPage Extensions are not installed under "
  echo "/usr/local/frontpage"
  exit
fi

# check we have permissions to write to that dir
touch /usr/local/frontpage/z_touch_check 2> /dev/null
if [ $? -ne 0 ]
then
  echo "FATAL:  this script needs to be run as root"
  exit
fi
rm /usr/local/frontpage/z_touch_check

# Ensure permissions in /usr/local/frontpage are reasonable
# Everything should be readable
find /usr/local/frontpage/ -print | xargs chmod +r 
# Anything that is executable, ensure it is also world executable
find /usr/local/frontpage/ -perm -100 -print | xargs chmod +x

# check ZEUSHOME is set
if [ -z "$ZEUSHOME" ]
then
  echo "Please set \$ZEUSHOME to point to the directory under which Zeus applications are installed"
  exit 1
fi

# get the virtual server name
while [ 1 ]
do
	printf "Enter virtual server name: "
	read SERVERNAME
	checkserver $SERVERNAME
	if [ $? -eq 0 ] 
	then
	  break
	fi
	printf "Invalid server name\n\n"
done


# get registry settings
getsetting port; PORT=$VALUE
getsetting ip_name; HOSTNAME=$VALUE
getsetting docroot; DOCROOT=$VALUE

# check the necessary modules are enabled
assert_moduleenabled frontpage
assert_moduleenabled htaccess 

# if the subserver module is enabled, then we'll ask them which softvserver
# they want to add frontpage to and alter the docroot to match

moduleenabled subserver
if [ $? -eq 0 ]
then
    echo "This server has subserver support enabled."
    while [ 1 ]
    do
	printf "Which subserver do you wish to enable FrontPage for? : "
	read SUBSERVERNAME
	if [ -d ${DOCROOT}/${SUBSERVERNAME} ] 
	then
	    break
	fi
	printf "Subserver does not exist\n\n"
    done
    FPSERVERNAME=${SERVERNAME}_${SUBSERVERNAME}
    HOSTNAME=$SUBSERVERNAME
    DOCROOT=${DOCROOT}/${SUBSERVERNAME}
else
  # Not using subserver, FPSERVERNAME is the same as SERVERNAME
  FPSERVERNAME=$SERVERNAME
fi


while [ 1 ] 
do
	printf "\nEnter UNIX user that owns the web: "
	read XUSER	
	checkuser $XUSER
	if [ $? -eq 0 ] 
	then
	  break
	fi
	printf "Invalid user name\n\n"
done


while [ 1 ]
do
	printf "Enter UNIX group that owns the web: "
	read XGROUP
	checkgroup $XGROUP
	if [ $? -eq 0 ] 
	then
	  break
	fi
	printf "Invalid group name\n\n"
done

echo

ADMIN=""
while [ -z "$ADMIN" ]
do
	printf "Enter the frontpage admin user name for the root web: "
	read ADMIN
done


PASSWD=""
getpassword "Enter password for the administrator: "
PASSWD=$VALUE


# OK - finally we have all the information we require.  Lets do some work

echo
ZFP=/usr/local/frontpage/zeus

# write out srm.conf
mkdir -p $ZFP/$FPSERVERNAME
chown $XUSER $ZFP/$FPSERVERNAME
chgrp $XGROUP $ZFP/$FPSERVERNAME
echo "Created directory $ZFP/$FPSERVERNAME"


# check the docroot exists
if [ ! -d "$DOCROOT" ]
then
    echo "Document Root $DOCROOT does not exist"
    exit
fi

# if its a symlink we must follow that symlink so that FP doesn't fail
# its security tests.  It won't ever write a file that is outside this 
# directory
if [ -h "$DOCROOT" ]
then
    NEWROOT=`ls -l "$DOCROOT" | sed 's|\(.*\)-> \(.*\)|\2|'`
    echo "Followed symlink from $DOCROOT to $NEWROOT"
    DOCROOT=$NEWROOT
fi


# write out a dumy srm.conf file that gives enough information about
# us so that the FrontPage CGIs know where we are

SRMCONF=$ZFP/$FPSERVERNAME/srm.conf
echo "DocumentRoot $DOCROOT" > $SRMCONF
echo "Port $PORT" >> $SRMCONF
echo "ServerRoot $ZFP/$FPSERVERNAME" >> $SRMCONF
echo '<VirtualHost '$HOSTNAME'>' >>  $SRMCONF
echo '</VirtualHost>' >>  $SRMCONF
chown $XUSER $SRMCONF
chgrp $XGROUP $SRMCONF
echo "Created file $ZFP/$FPSERVERNAME/srm.conf"

# create the symbolic link

cd $ZFP/$FPSERVERNAME
ln -sf . conf 2>/dev/null
echo "Created symlink $ZFP/$FPSERVERNAME/conf -> $ZFP/$FPSERVERNAME"


# run the Microsoft program to actually do the work.  But avoid their 
# interface.
# This program puts all the _vti files into the docroot.

echo "Running Microsoft fpsrvadm program:"
fpsrvadm.exe -o install -p $PORT -s $SRMCONF -u $ADMIN -pw $PASSWD -type apache-fp -xUser $XUSER -xGroup $XGROUP -multihost $HOSTNAME


